@@ -5,10 +5,5 @@ use base qw(example::BookStore::Object);
 use example::BookStore::BooksOrdered;
 use strict;
 
-BEGIN
-{
-	example::BookStore::Order->BindToObjectName( 'Order' );
-}
-
 1;
 
